/*-
 * ========================LICENSE_START=================================
 * Copyright (C) 2021 Nordix Foundation. All rights reserved.
 * ======================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ========================LICENSE_END===================================
 */

package org.oransc.rappmanager.configuration;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;

import javax.validation.constraints.NotEmpty;

import lombok.Getter;

import org.oransc.rappmanager.configuration.WebClientConfig.HttpProxyConfig;
import org.oransc.rappmanager.exception.ServiceException;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Component;

@Component
@ConfigurationProperties()
@EnableConfigurationProperties
public class ApplicationConfig {

    @Getter
    @Value("${app.vardata-directory}")
    private String vardataDirectory;

    @NotEmpty
    @Getter
    @Value("${app.certpath}")
    private String certPath;

    @NotEmpty
    @Getter
    @Value("${app.tokenpath}")
    private String tokenPath;

    @NotEmpty
    @Getter
    @Value("${app.basepath}")
    private String basePath;

    @NotEmpty
    @Getter
    @Value("${app.chartregistry}")
    private String chartApi;

    @Value("${server.ssl.key-store-type}")
    private String sslKeyStoreType = "";

    @Value("${server.ssl.key-store-password}")
    private String sslKeyStorePassword = "";

    @Value("${server.ssl.key-store}")
    private String sslKeyStore = "";

    @Value("${server.ssl.key-password}")
    private String sslKeyPassword = "";

    @Value("${app.webclient.trust-store-used}")
    private boolean sslTrustStoreUsed = false;

    @Value("${app.webclient.trust-store-password}")
    private String sslTrustStorePassword = "";

    @Value("${app.webclient.trust-store}")
    private String sslTrustStore = "";

    @Value("${app.webclient.http.proxy-host:\"\"}")
    private String httpProxyHost = "";

    @Value("${app.webclient.http.proxy-port:0}")
    private int httpProxyPort = 0;

    private WebClientConfig webClientConfig = null;

    public String getToken() throws ServiceException {
        try {
            Path path = Path.of(tokenPath);
            return Files.readString(path);
        } catch (IOException e) {
            throw new ServiceException("Error Reading token", e);
        }
    }

    public InputStream getCert() throws ServiceException {
        try {
            return new FileInputStream(certPath);
        } catch (IOException e) {
            throw new ServiceException("Error Reading token", e);
        }
    }

    public WebClientConfig getWebClientConfig() {
        if (this.webClientConfig == null) {
            HttpProxyConfig httpProxyConfig = ImmutableHttpProxyConfig.builder() //
                .httpProxyHost(this.httpProxyHost) //
                .httpProxyPort(this.httpProxyPort) //
                .build();

            this.webClientConfig = ImmutableWebClientConfig.builder() //
                .keyStoreType(this.sslKeyStoreType) //
                .keyStorePassword(this.sslKeyStorePassword) //
                .keyStore(this.sslKeyStore) //
                .keyPassword(this.sslKeyPassword) //
                .isTrustStoreUsed(this.sslTrustStoreUsed) //
                .trustStore(this.sslTrustStore) //
                .trustStorePassword(this.sslTrustStorePassword) //
                .httpProxyConfig(httpProxyConfig) //
                .build();
        }
        return this.webClientConfig;
    }

}
