
package org.oransc.rappmanager.controller;

import static org.assertj.core.api.Assertions.assertThat;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;

import java.io.File;
import java.lang.invoke.MethodHandles;
import java.util.Arrays;
import java.util.List;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.junit.runner.RunWith;
import org.oransc.rappmanager.BeanFactory;
import org.oransc.rappmanager.client.AsyncRestClient;
import org.oransc.rappmanager.client.AsyncRestClientFactory;
import org.oransc.rappmanager.configuration.ApplicationConfig;
import org.oransc.rappmanager.configuration.ImmutableWebClientConfig;
import org.oransc.rappmanager.configuration.WebClientConfig;
import org.oransc.rappmanager.exception.ServiceException;
import org.oransc.rappmanager.helm.HelmClient;
import org.oransc.rappmanager.service.App;
import org.oransc.rappmanager.service.AppStore;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.context.SpringBootTest.WebEnvironment;
import org.springframework.boot.test.context.TestConfiguration;
import org.springframework.boot.web.embedded.tomcat.TomcatServletWebServerFactory;
import org.springframework.boot.web.server.LocalServerPort;
import org.springframework.boot.web.servlet.server.ServletWebServerFactory;
import org.springframework.context.annotation.Bean;
import org.springframework.core.io.FileSystemResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.client.MultipartBodyBuilder;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit.jupiter.SpringExtension;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.BodyInserters.MultipartInserter;

@RunWith(SpringRunner.class)
@ExtendWith(SpringExtension.class)
@SpringBootTest(webEnvironment = WebEnvironment.RANDOM_PORT)
@TestPropertySource(
    properties = { //
        "server.ssl.key-store=./config/keystore.jks", //
        "app.webclient.trust-store=./config/truststore.jks", //
        "app.vardata-directory=./target" //
    })
public class KubernetesTest {
    private static final Logger logger = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
    private static Gson gson = new GsonBuilder().serializeNulls().create();
    private final static String APP_NAME = "test-application";
    private final static String VERSION = "1.0.0";
    private final static String NAMESPACE = "test-application-namespace";

    @LocalServerPort
    private int port;

    @Autowired
    ApplicationConfig applicationConfig;

    @Autowired
    AppStore appStore;

    static class MinikubeHelmClient extends HelmClient {
        public MinikubeHelmClient(AppStore appStore) {
            super(appStore);
        }

        @Override
        public ProcessBuilder prepareInstallCommand(App app) {
            String installCommand = toString(super.prepareInstallCommand(app));
            return executeInVagrant(installCommand);
        }

        @Override
        public ProcessBuilder prepareUnInstallCommand(App app) {
            String uninstallCommand = toString(super.prepareUnInstallCommand(app));
            return executeInVagrant(uninstallCommand);
        }

        @Override
        public ProcessBuilder prepareCreateNamespaceCommand(String namespace) {
            String createCommand = toString(super.prepareCreateNamespaceCommand(namespace));
            return executeInVagrant(createCommand);
        }

        @Override
        public ProcessBuilder prepareDeleteNamespaceCommand(String namespace) {
            String deleteCommand = toString(super.prepareDeleteNamespaceCommand(namespace));
            return executeInVagrant(deleteCommand);
        }

        private ProcessBuilder executeInVagrant(String command) {
            List<String> arguments = Arrays.asList("vagrant", //
                "ssh", //
                "--command", "sudo " + command);

            ProcessBuilder processBuilder = new ProcessBuilder();
            return processBuilder.command(arguments) //
                .directory(vagrantHome());
        }

        private File vagrantHome() {
            String vagrantHome = System.getenv("VAGRANT_HOME");
            if (vagrantHome.isEmpty()) {
                logger.error("VAGRANT_HOME not set");
            }
            return new File(vagrantHome);
        }
    }

    static class Microk8sHelmClient extends HelmClient {
        public Microk8sHelmClient(AppStore appStore) {
            super(appStore);
        }

        @Override
        public ProcessBuilder prepareInstallCommand(App app) {
            List<String> helmArguments = Arrays.asList("microk8s.helm", //
                "install", getChartFileName(app), //
                "--name", releaseName(app), //
                "--namespace", app.getNamespace(), //
                "--values", getOverrideFileName(app));

            ProcessBuilder processBuilder = new ProcessBuilder();
            return processBuilder.command(helmArguments);
        }

        @Override
        public ProcessBuilder prepareUnInstallCommand(App app) {
            List<String> helmArguments = Arrays.asList("microk8s.helm", "delete", releaseName(app), "--purge");
            ProcessBuilder processBuilder = new ProcessBuilder();
            return processBuilder.command(helmArguments);
        }
    }

    @TestConfiguration
    static class TestBeanFactory extends BeanFactory {

        @Bean
        @Override
        public ServletWebServerFactory servletContainer() {
            return new TomcatServletWebServerFactory();
        }

        @Bean
        @Override
        public HelmClient helmClient(@Autowired AppStore appStore) {
            // return new Microk8sHelmClient(appConfig);
            return new MinikubeHelmClient(appStore);
        }
    }

    @AfterEach
    @BeforeEach
    void reset() {
        appStore.reset();
    }

    @Test
    public void testPostApp_InstallApp_DeleteApp() throws ServiceException {

        onboardApp();
        App app = appStore.getApp(APP_NAME, VERSION);
        assertThat(app.getNamespace()).isEqualTo(NAMESPACE);

        installApp();

        uninstallApp();

        deleteApp();
    }

    @Test
    public void testStoreApp() throws ServiceException {
        appStore.reset();
        onboardApp();
        AppStore tmpAppStore = new AppStore(this.applicationConfig);
        App app = tmpAppStore.getApp(APP_NAME, VERSION);
        assertThat(app.getNamespace()).isEqualTo(NAMESPACE);
    }

    private void onboardApp() {
        String url = "/rms/apps";
        AppInfo appInfo = AppInfo.builder().name(APP_NAME).version(VERSION).namespace(NAMESPACE).build();

        File chartFile = new File(getClass().getClassLoader().getResource("testchart-0.1.0.tgz").getFile());
        File overrideFile = new File(getClass().getClassLoader().getResource("testValues.yaml").getFile());
        MultipartInserter body = createPostAppBody(chartFile, appInfo, overrideFile);
        ResponseEntity<String> e = restClient().postForEntity(url, body, MediaType.MULTIPART_FORM_DATA).block();

        assertThat(e.getStatusCode()).isEqualTo(HttpStatus.OK);
    }

    private void installApp() {
        InstallationInfo app = InstallationInfo.builder().name(APP_NAME).version(VERSION).build();
        String body = gson.toJson(app);
        String url = "/rms/install";
        ResponseEntity<String> e = restClient().postForEntity(url, body).block();
        assertThat(e.getStatusCode()).isEqualTo(HttpStatus.CREATED);
    }

    private void uninstallApp() {
        String url = "/rms/install/" + APP_NAME + "/" + VERSION;
        ResponseEntity<String> e = restClient().deleteForEntity(url).block();
        assertThat(e.getStatusCode()).isEqualTo(HttpStatus.NO_CONTENT);
    }

    private void deleteApp() {
        String url = "/rms/apps/" + APP_NAME + "/" + VERSION;
        ResponseEntity<String> e = restClient().deleteForEntity(url).block();
        assertThat(e.getStatusCode()).isEqualTo(HttpStatus.NO_CONTENT);
    }

    private MultipartInserter createPostAppBody(File chartFile, AppInfo appInfo, File overrideFile) {
        MultipartBodyBuilder bodyBuilder = new MultipartBodyBuilder();
        bodyBuilder.part("chart", new FileSystemResource(chartFile));
        bodyBuilder.part("values", new FileSystemResource(overrideFile));
        bodyBuilder.part("info", appInfo);
        return BodyInserters.fromMultipartData(bodyBuilder.build());
    }

    private AsyncRestClient restClient(String baseUrl, boolean useTrustValidation) {
        WebClientConfig config = this.applicationConfig.getWebClientConfig();
        config = ImmutableWebClientConfig.builder() //
            .keyStoreType(config.keyStoreType()) //
            .keyStorePassword(config.keyStorePassword()) //
            .keyStore(config.keyStore()) //
            .keyPassword(config.keyPassword()) //
            .isTrustStoreUsed(useTrustValidation) //
            .trustStore(config.trustStore()) //
            .trustStorePassword(config.trustStorePassword()) //
            .httpProxyConfig(config.httpProxyConfig()) //
            .build();

        AsyncRestClientFactory f = new AsyncRestClientFactory(config);
        return f.createRestClientNoHttpProxy(baseUrl);
    }

    private AsyncRestClient restClient(boolean useTrustValidation) {
        String baseUrl = "https://localhost:" + port;
        return restClient(baseUrl, useTrustValidation);
    }

    private AsyncRestClient restClient() {
        return restClient(false);
    }
}
